"""
 Author:  Huntly Cameron <huntly.cameron@gmail.com>

 Description:  Syncs podcast feeds to a local couchdb.  Specifically designed
               for ubuntu to use the ubuntu-one service.

 Notes:  Documentation for creating rhythmbox plugins is a bit thin on the 
         ground at the moment so I've left my initial development comments in 
         the code that describe in a very verbose way what's going on to 
         hopefully aid someone who finds him/herself in the same position that 
         I was in!

 Copyright (C) 2010  Huntly Cameron

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""

import rb
import gtk
import rhythmdb

have_desktop_couch = True
try:
    from desktopcouch.records.server import CouchDatabase
    from desktopcouch.records.record import Record
except ImportError:
    have_desktop_couch = False


ui_str = \
"""<ui>
  <menubar name="MenuBar">
    <menu name="ToolsMenu" action="Tools">
      <placeholder name="ToolsMenuModePlaceholder">
        <menuitem name="ToolsMenuPodcastSync" action="SyncPodcastFeeds"/>
      </placeholder>
    </menu>
  </menubar>
  <toolbar name="ToolBar">
    <placeholder name="ToolBarPluginPlaceholder">
      <toolitem name="U1 Podcast Sync" action="SyncPodcastFeeds"/>
    </placeholder>
  </toolbar>

  <popup name="PodcastSourcePopup">
    <placeholder name="PluginPlaceholder">
      <menuitem name="Sync Podcast Feeds" action="SyncPodcastFeeds"/>
    </placeholder>
  </popup>
  <popup name="PodcastViewPopup">
    <placeholder name="PluginPlaceholder">
      <menuitem name="Sync Podcast Feeds" action="SyncPodcastFeeds"/>
    </placeholder>
  </popup>
  <popup name="PodcastFeedViewPopup">
    <placeholder name="PluginPlaceholder">
      <menuitem name="Sync Podcast Feeds" action="SyncPodcastFeeds"/>
    </placeholder>
  </popup>
</ui>"""





class Podcast:
    def __init__(self, url, title, author, description):
        self.url = url
        self.title = title
        self.author = author
        self.description = description

class RBPodSync(rb.Plugin):
    def activate(self, shell):
        if not have_desktop_couch: 
            print "WARNING: You don't have a desktop couch!"
            return #plugin is useless without couchdb! Bail out!

        #We have a desktop couch, fire up the quattro 
        print "Activate!"

        # set up class level vars
        self.db = shell.get_property('db') #rhythmbox database object
        self.record_type = "http://huntlycameron.co.uk/rbpodcastformat/"	
        self.database_name = "rbpodcaststore"
        self.rb_podcast_type = "podcast-feed"
        self.local_podcasts = {}
        self.remote_podcasts = {}


        """UI creation taken from Stuart Langridge IRC lecture for Ubuntu 
           Opportunistic Development Week.  
           Code @ http://www.kryogenix.org/code/RBMicroBlog/
           Talk @ https://wiki.ubuntu.com/MeetingLogs/OpWeek1003/Rythmbox"""
        #Create a button with an image on it and hook it up to the sync method
        icon_file_name = "/usr/share/ubuntuone-client/icons/hicolor/24x24/" \
                         "status/ubuntuone-client-updating.png"
        iconsource = gtk.IconSource()
        iconsource.set_filename(icon_file_name)
        iconset = gtk.IconSet()
        iconset.add_source(iconsource)
        iconfactory = gtk.IconFactory()
        iconfactory.add("ubuntuone-button", iconset)
        iconfactory.add_default()
        action = gtk.Action("SyncPodcastFeeds", 
                            "Sync Podcast Feeds",
                            "Sync your podcast feeds to ubuntu-one", 
                            "ubuntuone-button")
        action.connect("activate", self.setup, shell)

        self.action_group = gtk.ActionGroup('SyncPodcastActionGroup')
        self.action_group.add_action(action)
        manager = shell.get_ui_manager()
        manager.insert_action_group(self.action_group, 0)
        self.UI_ID = manager.add_ui_from_string(ui_str)
        manager.ensure_update()

    def deactivate (self, shell):
        print "Deactivate!!"
        #cleanup the rhythmdb object
        self.db = None
        #cleanup ui
        manager = shell.get_ui_manager()
        manager.remove_ui(self.UI_ID)
        manager.remove_action_group(self.action_group)
        manager.ensure_update()


    def setup(self, event, shell):
        """Collects local podcasts and those from the couch before then going
           on to do the sync"""

        
        #call rhythmbox's entry_for_each_type(type, callback_method) .
        #The type can be found by running rhythmdb.entry_type_get_by_name() .
        #The callback method is add_to_local_podcast_list(entry) which
        #will append a new podcast with the relevant info to the 
        #local_podcast_list
        self.db.entry_foreach_by_type(
                          self.db.entry_type_get_by_name(self.rb_podcast_type), 
                          self.add_to_local_podcast_dictionary)

        self.remote_podcasts = self.populate_remote_podcast_dictionary()

        print "DEBUG -- list lengths:  local %s" % len(self.local_podcasts)
        print "DEBUG -- list lengths:  remote %s" % len(self.remote_podcasts)

        self.sync_podcast_feeds()
        self.clean_up()


    def add_to_local_podcast_dictionary(self, entry):
        """Add entry from rhythmbox and appends it to the local dictionary."""

        #set up some tmp vars to hold podcast info
        tmpTitle = self.db.entry_get(entry, rhythmdb.PROP_TITLE)
        tmpURI = self.db.entry_get(entry, rhythmdb.PROP_LOCATION)
        tmpAuthor = self.db.entry_get(entry, rhythmdb.PROP_ARTIST)
        tmpDescription = self.db.entry_get(entry, rhythmdb.PROP_DESCRIPTION)
        #If we don't have info for one of the fields then fill it in to avoid
        #issues with couchdb
        if tmpAuthor == None:
            tmpAuthor = "author"
        elif tmpDescription == None:
            tmpDescription = "description"
        #add podcast to the local_podcasts dictionary
        print "DEBUG -- Adding %s :: %s to lList" % (tmpTitle, tmpURI)
        self.local_podcasts[tmpURI] = Podcast(tmpURI,
                                              tmpTitle, 
                                              tmpAuthor, 
                                              tmpDescription)
        #dereference tmp variables
        tmpName = None
        tmpURI = None
        tmpTitle = None
        tmpAuthor = None
        tmpDescription = None

    def populate_remote_podcast_dictionary(self):
        """Collect all remote feeds from couchdb."""
        remote_podcasts = {} 		
        #get a ref to the couchdb: if the db doesn't exist, create it.	
        self.couchdb_database = CouchDatabase(self.database_name, create=True)		
        #Construct a view if it doesn't exist and use that to pull out all feeds
        results = self.couchdb_database.get_records(
                                                record_type = self.record_type, 		
                                                create_view = True)  		
        print "DEBUG -- Len Reulsts "
        print len(results)

        #Loop through the results and create Podcast objects to add to the
        #remote_podcasts list
        for result in results:
            podcast = result.value
            #set up some tmp vars to hold podcast info
            tmpURI = podcast["_id"]
            tmpTitle = podcast["title"]
            tmpAuthor = podcast["author"]
            tmpDescription = podcast["description"]
            #If we don't have info for one of the fields then fill it in to 
            #avoid issues with couchdb and rhythmbox 
            if tmpAuthor == None:
                tmpAuthor = "author"
            elif tmpDescription == None:
                tmpDescription = "description"
            #add podcast to the local_podcasts dictionary
            print "DEBUG -- Adding %s :: %s to lList" % (tmpTitle, tmpURI)
            remote_podcasts[podcast["_id"]] = Podcast(tmpURI,
                                                      tmpTitle, 
                                                      tmpAuthor, 
                                                      tmpDescription)
            #dereference tmp variables
            tmpName = None
            tmpURI = None
            tmpTitle = None
            tmpAuthor = None
            tmpDescription = None
               
            print """"DEBUG -- \b
                      added: %s :: %s\b
                      Author: %s\b
                      Description: %s\b
                      to rList""" % (podcast["title"], 
                                     podcast["_id"], 
                                     podcast["author"],
                                     podcast["description"])
        return remote_podcasts


    def sync_podcast_feeds(self):
        """Sync up the local podcast list with the remote podcast list."""
        print "DEBUG -- Made it into sync podcasts"

        #if we have at least one feed from couchdb, add to the local database
        if len(self.remote_podcasts) >= 1:                 
            #for each in couch and NOT in local, add to local
            for url, podcast in self.remote_podcasts.items():
                if url not in self.local_podcasts.keys():
                    print "DEBUG -- Local list has not got: %s :: %s" % (url, 
                                                                  podcast.title)
                    
                    entry = self.db.entry_new(
                                self.db.entry_type_get_by_name("podcast-feed"), 
                                url)
                    self.db.set(entry, rhythmdb.PROP_TITLE, podcast.title)
                    self.db.commit() #commit entry to the database
                    

                    self.db.set(entry, rhythmdb.PROP_ARTIST, podcast.author)
                    self.db.set(entry, 
                                rhythmdb.PROP_DESCRIPTION, 
                                podcast.description)

        #if we have at least one local feed, add to desktopcouch
        if len(self.local_podcasts) >= 1:
            #Get reference to couchdb
            self.couchdb_database = CouchDatabase(self.database_name, 
                                                  create=True)  
            #for each in local and NOT in couch, add to couch
            for url, podcast in self.local_podcasts.items():
                if url not in self.remote_podcasts.keys():
                    print "DEBUG -- Remote list has not got: %s :: %s" % (url, 
                                                                  podcast.title)

                    #Create a new couchdb record and add it to the couchdb
                    new_record = Record({"record_type": self.record_type,
                                         "_id": url,
                                         "title" : podcast.title,
                                         "author" : podcast.author, 
                                         "description" : podcast.description})
                    self.couchdb_database.put_record(new_record)         

            #call update podcast feeds
        print "DEBUG -- Sync complete!"

            
    def clean_up(self):
        """clear out local and remote podcast lists"""
        self.local_podcasts.clear() 
        self.remote_podcasts.clear()




    
